#!/bin/bash
# chknfs v1.5 - no longer failing silently on non Cent7 boxes
# Usage: ./chknfs.sh list.of.ip.addresses | tee results.txt
# Changelog:	1.0 - initial script
#		1.1 - improved
#		1.2 - displays folder contents
#		1.3 - can now use /mnt2/ without false positives
#		1.4 - detects dead netblocks and forwards through them
#		      also faster!
#		1.5 - Added compat. checks. This hasnt been tested on
#		      ubuntu.

iplist=$1

#constants
MAXFILES=100    #max files to display at once from one folder. Some people
		#have weird and huge folders...
MAXFAILURES=49  #max failed showmounts in a row for a /16, number is
		#approximate. This makes the scanner way faster.

usage()
{
	echo -n "Usage: $0 <file-with-ips>"
	echo ' | tee nfs_check.txt'
	exit
}

force_unmount()
{
	if [ -n "`mount | grep type\ nfs | grep \/mnt\ `" ] ; then
		umount -lf /mnt/
		sleep 0.5
	fi
}

list_dir()
{
	if [ -z $1 ] ; then
		echo "list_dir called without an arg"
	fi
	echo mount -t nfs -o nolock,soft,rw $nowip:$1 /mnt/
	timeout --foreground -s 9 10s mount -t nfs -o nolock,soft,rw $nowip:$1 /mnt/ 2>/dev/null
	if [ -n "`mount | grep type\ nfs | grep \/mnt\ `" ] ; then
		echo "-- $1 --"
		ls -a /mnt/ | head -$MAXFILES
		umount /mnt
	fi
	echo
	force_unmount
}

check_system_setup()
# not all timeouts have --foreground, which is sort of needed to save the data
# not all systems have dns utils
# i may as well check for /dev/shm. You can use anything, but /dev/shm doesn't
# wear out the disk (whoops). Nothing like having your fav host die of disk
# failure :(
{
	#this test will also be altered
	if [ -n "`timeout --foreground 2>&1 | grep --color 'unrecognized option'`" ] ;
	then
		echo -n 'Run '
		echo sed -i 's/timeout\ \-\-foreground/timeout\ /g' $0
		echo "to remove a flag some linux distros don't like. To stop"
		echo "the script after doing this, use ctrl+z then kill the"
		echo "process once it's been stopped."
		exit 1
	fi

	if [ -z "`id | grep 'uid=0'`" ] ; then
		echo "This script needs to be run as root"
		exit 1
	fi	

	for util in mount umount showmount tee sed grep cat ; do
		if [ ! -x "`which $util`" ] ; then
			echo "Missing needed binary $util, install nfs-utils"
			exit
		fi
	done

	if [ ! -d /dev/shm ] ; then
		echo "By default chknfs uses /dev/shm - a default ramdisk."
		echo "It's to avoid breaking hard disks that are about to die anyway."
		echo "Use sed to change the directory if it doesn't exist: "
		echo sed -i 's/\/dev\/shm/\/tmp/g' $0
		exit 1
	fi

	if [ ! -w ./ ] ; then 
		echo "Your local directory isn't writable."
		echo 'chmod u+w ./'
		exit 1
	fi
}

#### main ######
if [ -z $1 ] ; then
	usage
fi
check_system_setup

# for keeping track of bad netblocks
last16=""
numfailures=0

for nowip in `cat $iplist | grep -E -o '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}'  ` ; do
	echo $nowip
	#hurry through bad ips
	cur16="`echo $nowip | cut -d . -f 1,2`"
	if [ $numfailures -ge $MAXFAILURES -a "$last16" = "$cur16" ] ; then
		continue
	fi

	echo > /dev/shm/.showmount
	#echo $nowip
	timeout --foreground -s 9 5s showmount -e $nowip >/dev/shm/.showmount 2>/dev/null
	if [ -s '/dev/shm/.showmount' ] ; then
		#set failed showmount count to 0
		numfailures=0
		
		cat /dev/shm/.showmount
		result="$(cat /dev/shm/.showmount | grep -e '*' -e everyone | awk '{print $1}' | grep ^\/)"
		for export in $result ; do 
			list_dir "$export"
		done

		#grab the unlabeled exports
		result="$(cat /dev/shm/.showmount | awk '{print $NF}' | grep -e ^/ )"
		for unlabeled in $result ; do
			list_dir "$unlabeled"
		done
	else
		#echo "Host ($nowip) did not respond."
		#code to count failures per /16
		cur16="`echo $nowip | cut -d . -f 1,2`"
                if [ "$last16" = "$cur16" ] ; then
                        ((numfailures++))
                fi
		last16="$cur16"
	fi
done

rm -f /dev/shm/.showmount
echo "Done?"
